package api

import (
	"3dshow-supplier/bus"
	"3dshow-supplier/db/model"
	"3dshow-supplier/db/repo"
	"3dshow-supplier/funcgraph"
	"time"

	_ "image/jpeg"
	_ "image/png"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func regAssetApi(r *GinRouter) {

	// 添加管理端接口
	CreateCRUD(r, "/assets", &CRUDOption{
		Collection: repo.CollectionAssets,
		NewModel: func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
			entity := &model.Asset360Fake3d{}
			c.ShouldBindJSON(entity)

			entity.SupplyId, _ = primitive.ObjectIDFromHex(apictx.User.Parent)
			entity.CreateTime = time.Now()

			if entity.Groups != nil {
				for _, g := range entity.Groups {
					if len(g.ImagesPath) < 1 {
						g.State = model.GROUPSTATE_UNSET
					} else {
						g.State = model.GROUPSTATE_ADDED
					}
				}
			}
			return entity, nil
		},

		EmtyModel: func(c *gin.Context, apictx *ApiSession) interface{} {
			return &model.Asset360Fake3d{}
		},
		SearchSort: bson.M{"createTime": -1},
		SearchFilter: func(c *gin.Context, apictx *ApiSession, query map[string]interface{}) map[string]interface{} {
			if query["supplyId"] != nil {
				query["supplyId"], _ = primitive.ObjectIDFromHex(query["supplyId"].(string))
			}
			return query
		},
		JWT:           true,
		SearchProject: []string{"name", "createTime", "type", "unit", "price", "cover", "status"},
		OnUpdate: func(c *gin.Context, apictx *ApiSession, entity interface{}, id primitive.ObjectID) {
			asset := entity.(*model.Asset360Fake3d)
			if asset == nil {
				return
			}
			natsUrl := ""
			if asset.Groups == nil {
				return
			}

			for _, g := range asset.Groups {
				if len(g.ImagesPath) < 1 {
					g.State = model.GROUPSTATE_UNSET
				} else {
					if len(g.State) < 1 {
						g.State = model.GROUPSTATE_ADDED
					}
				}

				if g.State == model.GROUPSTATE_ADDED && len(g.ImagesPath) > 1 {
					if len(natsUrl) < 1 {
						url, err := bus.NatsCenter.RequestConfig("bus-network")
						if err != nil {
							g.State = model.GROUPSTATE_FAIL
							return
						}
						natsUrl = url
					}
					err := funcgraph.Tile360onvRequest(id.Hex(), g.Id, g.ImagesPath, natsUrl)
					if err != nil {
						g.State = model.GROUPSTATE_FAIL
					} else {
						g.State = model.GROUPSTATE_CROPING
					}
				}
			}
		},
	})
}
